@@ -4,19 +4,15 @@ from json import JSONDecodeError
 from logging import getLogger
 from typing import List, Union
 
-from aiohttp import ClientError, ClientSession
-from switchbee.device import SwitchBeeGroupSwitch
-from switchbee.device import (
-    DeviceType,
-    HardwareType,
-    SwitchBeeDimmer,
-    SwitchBeeScenario,
-    SwitchBeeShutter,
-    SwitchBeeSwitch,
-    SwitchBeeTimerSwitch,
-)
+from aiohttp import ClientSession
+from device import ThermostatMode
 
 from switchbee.const import ApiAttribute, ApiCommand, ApiStatus
+from switchbee.device import (DeviceType, HardwareType, SwitchBeeDimmer,
+                              SwitchBeeGroupSwitch, SwitchBeeScenario,
+                              SwitchBeeShutter, SwitchBeeSwitch,
+                              SwitchBeeThermostat, SwitchBeeTimerSwitch)
+
 from .utils import timestamp_now
 
 logger = getLogger(__name__)
@@ -34,7 +30,7 @@ class SwitchBeeDeviceOfflineError(Exception):
     pass
 
 
-TOKEN_EXPIRATION = int(timedelta(minutes=50).total_seconds()) * 1000
+TOKEN_EXPIRATION = int(timedelta(minutes=55).total_seconds()) * 1000
 
 
 class CentralUnitAPI:
@@ -45,7 +41,7 @@ class CentralUnitAPI:
         self._user: str = user
         self._password: str = password
         self._session: ClientSession = websession
-        self._token: str = ""
+        self._token: str = None
         self._token_expiration: int = 0
         self._login_count: int = -1  # we don't count the first login
         self._devices_map: dict[
@@ -56,6 +52,8 @@ class CentralUnitAPI:
                 SwitchBeeShutter,
                 SwitchBeeScenario,
                 SwitchBeeTimerSwitch,
+                SwitchBeeGroupSwitch,
+                SwitchBeeThermostat,
             ],
         ] = {}
         self._mac = str
@@ -93,6 +91,7 @@ class CentralUnitAPI:
             SwitchBeeShutter,
             SwitchBeeTimerSwitch,
             SwitchBeeScenario,
+            SwitchBeeThermostat,
         ]
     ]:
         return self._devices_map.values()
@@ -101,7 +100,15 @@ class CentralUnitAPI:
     def reconnect_count(self) -> int:
         return self._login_count
 
-    async def connect(self):
+    async def login_if_needed(self) -> None:
+        if not self._token or (timestamp_now() >= self._token_expiration):
+            logger.info(
+                "Logging into the Central Unit due to %s",
+                "invalid token" if not self._token else " expiry",
+            )
+            await self._login()
+
+    async def connect(self) -> None:
         await self.fetch_configuration()
         await self.fetch_states()
 
@@ -121,6 +128,7 @@ class CentralUnitAPI:
                                 ApiStatus.INVALID_TOKEN,
                                 ApiStatus.TOKEN_EXPIRED,
                             ]:
+                                self._token = None
                                 raise SwitchBeeTokenError(
                                     json_result[ApiAttribute.STATUS]
                                 )
@@ -141,21 +149,12 @@ class CentralUnitAPI:
                     raise SwitchBeeError(
                         f"Request to the Central Unit failed with status={response.status}"
                     )
-
         except TimeoutError:
-            raise SwitchBeeError(f"Timeout error: {response}")
-        except ClientError as e:
-            raise SwitchBeeError(f"Client error: {e}")
+            raise SwitchBeeError(
+                f"Timed out while waiting for the Central Unit to reply"
+            )
 
     async def _send_request(self, command: ApiCommand, params: dict = {}) -> dict:
-        if not self._token or (
-            self._token and timestamp_now() >= self._token_expiration
-        ):
-            logger.info(
-                "Logging into the Central Unit %s",
-                "for the first time." if not self._token else " due to Token expiry.",
-            )
-            await self._login()
 
         return await self._post(
             {
@@ -192,24 +191,29 @@ class CentralUnitAPI:
         return self._token
 
     async def get_configuration(self):
+        await self.login_if_needed()
         return await self._send_request(ApiCommand.GET_CONF)
 
     async def get_multiple_states(self, ids: list):
         """returns JSON {'status': 'OK', 'data': [{'id': 212, 'state': 'OFF'}, {'id': 343, 'state': 'OFF'}]}"""
+        await self.login_if_needed()
         return await self._send_request(ApiCommand.GET_MULTI_STATES, ids)
 
     async def get_state(self, id: int):
         """returns JSON {'status': 'OK', 'data': 'OFF'}"""
+        await self.login_if_needed()
         return await self._send_request(ApiCommand.GET_STATE, id)
 
     async def set_state(self, id: int, state):
         """returns JSON {'status': 'OK', 'data': 'OFF/ON'}"""
+        await self.login_if_needed()
         return await self._send_request(
             ApiCommand.OPERATE, {"directive": "SET", "itemId": id, "value": state}
         )
 
     async def get_stats(self):
         """returns {'status': 'OK', 'data': {}} on my unit"""
+        await self.login_if_needed()
         return await self._send_request(ApiCommand.STATS)
 
     async def fetch_configuration(
@@ -219,8 +223,10 @@ class CentralUnitAPI:
             DeviceType.Dimmer,
             DeviceType.TimedPowerSwitch,
             DeviceType.Shutter,
+            DeviceType.Thermostat,
         ],
     ):
+        await self.login_if_needed()
         data = await self.get_configuration()
         if data[ApiAttribute.STATUS] != ApiStatus.OK:
             raise SwitchBeeError
@@ -294,7 +300,18 @@ class CentralUnitAPI:
                         hardware=HardwareType(device_hw),
                         type=DeviceType.GroupSwitch,
                     )
-
+                elif device_type == DeviceType.Thermostat.value:
+                    self._devices_map[item[ApiAttribute.ID]] = SwitchBeeThermostat(
+                        id=item[ApiAttribute.ID],
+                        name=item[ApiAttribute.NAME],
+                        zone=zone[ApiAttribute.NAME],
+                        hardware=HardwareType(device_hw),
+                        type=DeviceType.Thermostat,
+                        modes=[
+                            ThermostatMode(mode) for mode in item[ApiAttribute.MODES]
+                        ],
+                        unit=item[ApiAttribute.TEMPERATURE_UNITS],
+                    )
                 else:
                     logger.warning(f"Unknown Type {item[ApiAttribute.TYPE]}")
 
@@ -314,6 +331,7 @@ class CentralUnitAPI:
                     DeviceType.Dimmer,
                     DeviceType.Shutter,
                     DeviceType.TimedPowerSwitch,
+                    DeviceType.Thermostat,
                 ]
             ]
         )
@@ -332,3 +350,19 @@ class CentralUnitAPI:
                 self._devices_map[device_id].state = device_state[ApiAttribute.STATE]
             elif self._devices_map[device_id].type == DeviceType.GroupSwitch:
                 self._devices_map[device_id].state = device_state[ApiAttribute.STATE]
+            elif self._devices_map[device_id].type == DeviceType.Thermostat:
+                self._devices_map[device_id].state = device_state[ApiAttribute.STATE][
+                    ApiAttribute.POWER
+                ]
+                self._devices_map[device_id].mode = device_state[ApiAttribute.STATE][
+                    ApiAttribute.MODE
+                ]
+                self._devices_map[device_id].fan = device_state[ApiAttribute.STATE][
+                    ApiAttribute.FAN
+                ]
+                self._devices_map[device_id].target_temperature = device_state[
+                    ApiAttribute.STATE
+                ][ApiAttribute.CONFIGURED_TEMPERATURE]
+                self._devices_map[device_id].temperature = device_state[
+                    ApiAttribute.STATE
+                ][ApiAttribute.ROOM_TEMPERATURE]
